from sdk32 import *

bitmap = 0
xclient = yclient = xpixel = ypixel = xcenter = ycenter = 0
xradius = yradius = xmove = ymove = xtotal = ytotal = 0

def wnd_proc(wnd, msg, wparam, lparam):
    global bitmap, xclient, yclient, xpixel, ypixel, xcenter, ycenter, \
           xradius, yradius, xmove, ymove, xtotal, ytotal
    if msg == WM_CREATE:
        dc = GetDC(wnd)
        xpixel = GetDeviceCaps(dc, ASPECTX)
        ypixel = GetDeviceCaps(dc, ASPECTY)
        ReleaseDC(wnd, dc)
        return 0

    elif msg == WM_SIZE:
        xclient = LOWORD(lparam)
        yclient = HIWORD(lparam)
        xcenter = xclient / 2
        ycenter = yclient / 2

        scale = min(xclient * xpixel, yclient * ypixel) / 16

        xradius = scale / xpixel
        yradius = scale / ypixel
        xmove = max(1, xradius / 4)
        ymove = max(1, yradius / 4)

        xtotal = 2 * (xradius + xmove)
        ytotal = 2 * (yradius + ymove)

        if bitmap:
            DeleteObject(bitmap)

        dc = GetDC(wnd)
        mem_dc = CreateCompatibleDC(dc)
        bitmap = CreateCompatibleBitmap(dc, xtotal, ytotal)
        ReleaseDC(wnd, dc)

        SelectObject(mem_dc, bitmap)
        Rectangle(mem_dc, -1, -1, xtotal + 1, ytotal + 1)

        brush = CreateHatchBrush(HS_DIAGCROSS, 0)
        SelectObject(mem_dc, brush)
        SetBkColor(mem_dc, RGB(255, 0, 255))
        Ellipse(mem_dc, xmove, ymove, xtotal - xmove, ytotal - ymove)

        DeleteDC(mem_dc)
        DeleteObject(brush)
        return 0

    elif msg == WM_TIMER:
        if not bitmap:
            return DefWindowProc(wnd, msg, wparam, lparam)
        dc = GetDC(wnd)
        mem_dc = CreateCompatibleDC(dc)
        SelectObject(mem_dc, bitmap)

        BitBlt(dc, xcenter - xtotal / 2, ycenter - ytotal / 2, xtotal, ytotal,
               mem_dc, 0, 0, SRCCOPY)

        ReleaseDC(wnd, dc)
        DeleteDC(mem_dc)

        xcenter += xmove
        ycenter += ymove
        if xcenter + xradius >= xclient or xcenter - xradius < 0:
            xmove = -xmove
        if ycenter + yradius >= yclient or ycenter - yradius < 0:
            ymove = -ymove
        return 0

    elif msg == WM_DESTROY:
        if bitmap:
            DeleteObject(bitmap)
        KillTimer(wnd, 1)
        PostQuitMessage(0)
        return 0

    return DefWindowProc(wnd, msg, wparam, lparam)

def winmain(inst, prev_inst, cmd_line, cmd_show):
    app_name = 'Bounce'

    wc = WNDCLASSEX()
    wc.style = CS_HREDRAW | CS_VREDRAW
    wc.wnd_proc = wnd_proc
    wc.inst = inst
    wc.cursor = LoadCursor(0, IDC_ARROW)
    wc.background = GetStockObject(WHITE_BRUSH)
    wc.name = app_name
    RegisterClassEx(wc)

    wnd = CreateWindow(app_name, 'Bouncing Ball', WS_OVERLAPPEDWINDOW,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, inst, 0)
    if not SetTimer(wnd, 1, 50, None):
        MessageBox(wnd, 'Too many clocks or timers!', app_name,
                   MB_ICONEXCLAMATION | MB_OK)
        return FALSE

    ShowWindow(wnd, cmd_show)
    UpdateWindow(wnd)

    msg = MSG()
    while 1:
        if not GetMessage(msg, 0, 0, 0):
            break
        TranslateMessage(msg)
        DispatchMessage(msg)
    return msg.wparam

if __name__ == '__main__':
    winmain(GetModuleHandle(None), 0, '', SW_SHOWNORMAL)
