from sdk32 import *

sysmetrics = (
    (SM_CXSCREEN,      'SM_CXSCREEN',      'Screen width in pixels' ),
    (SM_CYSCREEN,      'SM_CYSCREEN',      'Screen height in pixels' ),
    (SM_CXVSCROLL,     'SM_CXVSCROLL',     'Vertical scroll arrow width' ),
    (SM_CXHSCROLL,     'SM_CXHSCROLL',     'Horizontal scroll arrow height' ),
    (SM_CYCAPTION,     'SM_CYCAPTION',     'Caption bar height' ),
    (SM_CXBORDER,      'SM_CXBORDER',      'Border width' ),
    (SM_CYBORDER,      'SM_CYBORDER',      'Border height' ),
    (SM_CXDLGFRAME,    'SM_CXDLGFRAME',    'Dialog window frame width' ),
    (SM_CYDLGFRAME,    'SM_CYDLGFRAME',    'Dialog window frame height' ),
    (SM_CYVTHUMB,      'SM_CYVTHUMB',      'Vertical scroll thumb height' ),
    (SM_CXHTHUMB,      'SM_CXHTHUMB',      'Horizontal scroll thumb width' ),
    (SM_CXICON,        'SM_CXICON',        'Icon width' ),
    (SM_CYICON,        'SM_CYICON',        'Icon height' ),
    (SM_CXCURSOR,      'SM_CXCURSOR',      'Cursor width' ),
    (SM_CYCURSOR,      'SM_CYCURSOR',      'Cursor height' ),
    (SM_CYMENU,        'SM_CYMENU',        'Menu bar height' ),
    (SM_CXFULLSCREEN,  'SM_CXFULLSCREEN',  'Full-screen client window width' ),
    (SM_CYFULLSCREEN,  'SM_CYFULLSCREEN',  'Full-screen client window height' ),
    (SM_CYKANJIWINDOW, 'SM_CYKANJIWINDOW', 'Kanji window height' ),
    (SM_MOUSEPRESENT,  'SM_MOUSEPRESENT',  'Mouse present flag' ),
    (SM_CYVSCROLL,     'SM_CYVSCROLL',     'Vertical scroll arrow height' ),
    (SM_CXHSCROLL,     'SM_CXHSCROLL',     'Horizontal scroll arrow width' ),
    (SM_DEBUG,         'SM_DEBUG',         'Debug version flag' ),
    (SM_SWAPBUTTON,    'SM_SWAPBUTTON',    'Mouse buttons swapped flag' ),
    (SM_RESERVED1,     'SM_RESERVED1',     'Reserved' ),
    (SM_RESERVED2,     'SM_RESERVED2',     'Reserved' ),
    (SM_RESERVED3,     'SM_RESERVED3',     'Reserved' ),
    (SM_RESERVED4,     'SM_RESERVED4',     'Reserved' ),
    (SM_CXMIN,         'SM_CXMIN',         'Minimum window width' ),
    (SM_CYMIN,         'SM_CYMIN',         'Minimum window height' ),
    (SM_CXSIZE,        'SM_CXSIZE',        'Minimize/Maximize icon width' ),
    (SM_CYSIZE,        'SM_CYSIZE',        'Minimize/Maximize icon height' ),
    (SM_CXFRAME,       'SM_CXFRAME',       'Window frame width' ),
    (SM_CYFRAME,       'SM_CYFRAME',       'Window frame height' ),
    (SM_CXMINTRACK,    'SM_CXMINTRACK',    'Minimum tracking width of window' ),
    (SM_CYMINTRACK,    'SM_CYMINTRACK',    'Minimum tracking height of window' ),
    (SM_CMETRICS,      'SM_CMETRICS',      'Number of system metrics' ),
)

tm = TEXTMETRIC()
ps = PAINTSTRUCT()
rect = RECT()
xchar = xcaps = ychar = 0

def wnd_proc(wnd, msg, wparam, lparam):
    global tm, ps, rect, xchar, xcaps, ychar
    if msg == WM_CREATE:
        dc = GetDC(wnd)
        GetTextMetrics(dc, tm)
        xchar = tm.ave_char_width
        if tm.pitch_and_family & 1:
            xcaps = 3 * xchar / 2
        else:
            xcaps = 2 * xchar / 2
        ychar = tm.height + tm.external_leading
        ReleaseDC(wnd, dc)
        return 0

    elif msg == WM_PAINT:
        dc = BeginPaint(wnd, ps)
        line = 0
        for metric, label, desc in sysmetrics:
            line = line + 1
            TextOut(dc, xchar, ychar * line, label, len(label))
            TextOut(dc, xchar + 18 * xcaps, ychar * line, desc, len(desc))
            SetTextAlign(dc, TA_RIGHT | TA_TOP)
            buff = '%s' % GetSystemMetrics(metric)
            TextOut(dc, xchar + 18 * xcaps + 40 * xchar, ychar * line,
                    buff, len(buff))
            SetTextAlign(dc, TA_LEFT | TA_TOP)
        EndPaint(wnd, ps)
        return 0

    elif msg == WM_DESTROY:
        PostQuitMessage(0)
        return 0

    return DefWindowProc(wnd, msg, wparam, lparam)

def winmain(inst, prev_inst, cmd_line, cmd_show):
    app_name = 'SysMets1'

    wc = WNDCLASSEX()
    wc.style = CS_HREDRAW | CS_VREDRAW
    wc.wnd_proc = wnd_proc
    wc.inst = inst
    wc.cursor = LoadCursor(0, IDC_ARROW)
    wc.background = GetStockObject(WHITE_BRUSH)
    wc.name = app_name
    RegisterClassEx(wc)

    wnd = CreateWindow(app_name, 'Get System Metrics No. 1',
                       WS_OVERLAPPEDWINDOW,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, inst, 0)

    ShowWindow(wnd, cmd_show)
    UpdateWindow(wnd)

    msg = MSG()
    while 1:
        if not GetMessage(msg, 0, 0, 0):
            break
        TranslateMessage(msg)
        DispatchMessage(msg)
    return msg.wparam

if __name__ == '__main__':
    winmain(GetModuleHandle(None), 0, '', SW_SHOWNORMAL)
